==> loloswitcher_0_15.tar.gz: loloswitcher_0_15/Config.cpp <==
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

#include "Config.h"
#include "ParseHelper.h"


Config::Config()
{
 init();
}


Config::~Config()
{

}


void Config::init(void)
{
 //    
 sprintf(configFileName, "");


 // ------------------------
 //   
 // ------------------------

 //   
 sprintf(inputDevice, "");

 //   
 numberOfLayout=0;

 //  
 switchMethod=0;

 //  
 sprintf(eventFilter.regexp, "");
 memset(eventFilter.regexpCompile, 0, REGEXP_COMPILE_SIZE);

 for(int i=0; i<CODES_LAYOUT_NUMBER; i++)
  {
   //    
   sprintf(sequences[i].regexp, "");
   memset(sequences[i].regexpCompile, 0, REGEXP_COMPILE_SIZE);

   //   
   sprintf(commands[i].cmdText, "");
  }
}


//   
bool Config::readFile(char *fileName)
{
 init();

 //  
 FILE *uk;
 if((uk = fopen (fileName, "rt")) == NULL) //     
  return false;

 //      

 //   
 sprintf(configFileName, fileName);


 //  
 char readline[255];
 char tmpline[255];
 char tmpline2[255];
 int  i;
 bool remflag,eqflag;

 //    
 while(!feof(uk)){

  //  
  readline[0]='\0';

  //  
  fgets(readline, 250, uk);
  alltrim(readline);

  // ,     
  remflag=false;
  if(readline[0]=='#')
   remflag=true;

  //   
  eqflag=false;
  for(i=0; i<=strlen(readline); i++)
   if(readline[i]=='=')
    eqflag=true;

  //        
  if(eqflag==true && remflag!=true)
  {
   //   readline
   for(i=0; readline[i]!='\0'; i++)
    tmpline[i]=readline[i];
   tmpline[i]='\0';
   getparamname(tmpline);

   //   readline
   for(i=0;readline[i]!='\0';i++)
    tmpline2[i]=readline[i];
   tmpline2[i]='\0';
   getparamvalue(tmpline2);

   /*
   printf("readline: %s\n", readline);
   printf("tmpline : %s\n", tmpline);
   printf("tmpline2: %s\n", tmpline2);
   */

   //   InputDevice
   if(strcmp(tmpline, "InputDevice")==0)
    {
     sprintf(inputDevice, "%s", tmpline2);

     if(strlen(inputDevice)==0)
      {
       printf("Detect empty value for inputDevice\n");
       exit(1);
      }
    }


   //   DeviceType
   if(strcmp(tmpline, "DeviceType")==0)
    {
     deviceType=atoi(tmpline2);

     if(deviceType<0 || deviceType>1)
      {
       printf("Illegal value for DeviceType: %d\n", deviceType);
       exit(1);
      }
    }

   //   NumberOfLayout
   if(strcmp(tmpline, "NumberOfLayout")==0)
    {
     numberOfLayout=atoi(tmpline2);

     if(numberOfLayout<1 || numberOfLayout>CODES_LAYOUT_NUMBER)
      {
       printf("Illegal value for NumberOfLayout: %d\n", numberOfLayout);
       exit(1);
      }
    }


   //   SwitchMethod
   if(strcmp(tmpline, "SwitchMethod")==0)
    {
     switchMethod=atoi(tmpline2);

     if(switchMethod<0 || switchMethod>1)
      {
       printf("Illegal value for switchMethod: %d\n", switchMethod);
       exit(1);
      }  
    }


   //   EventFilter
   if(strcmp(tmpline, "EventFilter")==0)
    {
     //    
     sprintf(eventFilter.regexp, tmpline2);

     //   
     regexpCompiling(eventFilter.regexp, eventFilter.regexpCompile);
    }


   //   Sequence  Command
   for(int n=0; n<CODES_LAYOUT_NUMBER && n<numberOfLayout; n++)
    {
     //   
     char varName[250];
     sprintf(varName, "Sequence%d", n);

     //   Sequence
     if(strcmp(tmpline, varName)==0)
      {
       //    
       sprintf(sequences[n].regexp, tmpline2);

       //   
       regexpCompiling(sequences[n].regexp, sequences[n].regexpCompile);
      }


     //   Command
     sprintf(varName, "Command%d", n);

     //  
     if(strcmp(tmpline, varName)==0)
      sprintf(commands[n].cmdText, tmpline2); //   

    } //     Sequence
    
  }
 };

 fclose(uk);

 return true;

}

//      
void Config::regexpCompiling(char *regexpText, char *regexpCompileData)
{
 int options = 0;
 const char *error;
 int erroffset;
 pcre *regexp;
 
 regexp=pcre_compile(regexpText, options, &error, &erroffset, NULL);

 //    
 if(!regexp)
  {
   printf("Problem RegExp compile: '%s'\n", regexpText);
   printf("Error: %s\n", error);
   exit(1);
  }


 //     
 int size, sizeResult;
 sizeResult = pcre_fullinfo(regexp, NULL, PCRE_INFO_SIZE, &size);

 if(sizeResult<0)
  {
   printf("Error if call pcre_fullinfo() for get size, %s\n", regexpText);
   exit(1);
  }

 if(size>=REGEXP_COMPILE_SIZE)
  {
   printf("Compile data for regexp too long, %d, %s\n", size, regexpText);
   exit(1);
  }

 //      
 memcpy(regexpCompileData, (char *)regexp, size);
}


// ,     
void Config::print(void)
{
 printf("\n");
 printf("-------------------\n");
 printf("Reading config data from file %s\n", configFileName);
 printf("-------------------\n");

 printf("InputDevice: %s\n", inputDevice);
 printf("DeviceType: %d\n", deviceType);
 printf("NumberOfLayout: %d\n", numberOfLayout);
 printf("SwitchMethod: %d\n", switchMethod);
 printf("EventFilter: %s\n", eventFilter.regexp);

 //   
 for(int n=0; n<CODES_LAYOUT_NUMBER && n<numberOfLayout; n++)
  printf("Sequence%d: %s\n", n, sequences[n].regexp);

 //   
 for(int n=0; n<CODES_LAYOUT_NUMBER && n<numberOfLayout; n++)
  printf("Command%d: %s\n", n, commands[n].cmdText);

 printf("\n");
}


//    
void Config::createStandartConfig()
{
 char dirName[250];

 sprintf(dirName, "%s/.config", getUserDirectory());
 createDirIfNotExists(dirName, getUserName());

 sprintf(dirName, "%s/.config/loloswitcher", getUserDirectory());
 createDirIfNotExists(dirName, getUserName());

 //    
 char fileName[250];
 sprintf(fileName, "%s/.config/loloswitcher/config.ini", getUserDirectory());

 FILE *uk;
 if((uk = fopen (fileName, "wt")) == NULL)
  {
   //     
   printf("Error! Can not create file %s.\n", fileName);
   return;
  }


 //   
 fputs("# LoLo Switcher config file\n",                                            uk);
 fputs("\n",                                                                       uk);
 fputs("# ConfigVersion (do not edit this parameter!)\n",                          uk);
 fputs("ConfigVersion=1\n",                                                        uk);
 fputs("\n",                                                                       uk);
 fputs("# Input device\n",                                                         uk);
 fputs("InputDevice=/dev/input/event0\n",                                          uk);
 fputs("\n",                                                                       uk);
 fputs("# Type of device\n",                                                       uk);
 fputs("# 0 - keyboard\n",                                                         uk);
 fputs("# 1 - joystick\n",                                                         uk);
 fputs("DeviceType=0\n",                                                           uk);
 fputs("\n",                                                                       uk);
 fputs("# Total number of language layout\n",                                      uk);
 fputs("# For example:\n",                                                         uk);
 fputs("# if you use ENG and RUS, set 2\n",                                        uk);
 fputs("# if you use ENG, RUS and UKR, set 3\n",                                   uk);
 fputs("NumberOfLayout=2\n",                                                       uk);
 fputs("\n",                                                                       uk);
 fputs("# Language switch method\n",                                               uk);
 fputs("# 0 - cyclic switch\n",                                                    uk);
 fputs("# 1 - direct switch\n",                                                    uk);
 fputs("SwitchMethod=1\n",                                                         uk);
 fputs("\n",                                                                       uk);
 fputs("# RegExp for primary filtering device events\n",                           uk);
 fputs("# For classic keyboard, set value to ^1,[0-9]+,[0-9]+;$\n",                uk);
 fputs("# and this filter enabled only KeyPress, KeyHold and KeyRelease event.\n", uk);
 fputs("# Before research device codes (with option -t1), clear this value.\n",    uk);
 fputs("EventFilter=^1,[0-9]+,[0-9]+;$\n",                                         uk);
 fputs("\n",                                                                       uk);
 fputs("# RegExp with codes for switch language layouts\n",                        uk);
 fputs("# If you use cyclic switch method, set variable Sequence0 only.\n",        uk);
 fputs("# If you use direct switch method, set variable SequenceX\n",              uk);
 fputs("# for each layout (numeric from 0).\n",                                    uk);
 fputs("Sequence0=(?<!1,29,1;|1,29,2;|1,97,1;|1,97,2;|1,56,1;|1,56,2;|1,100,1;|1,100,2;)1,42,1;1,42,0;$\n", uk);
 fputs("Sequence1=(?<!1,29,1;|1,29,2;|1,97,1;|1,97,2;|1,56,1;|1,56,2;|1,100,1;|1,100,2;)1,54,1;1,54,0;$\n", uk);
 fputs("\n",                                                                       uk);
 fputs("# Bash command if language layout switched (optional).\n",                 uk);
 fputs("# Set CommandX for each layout (numeric from 0).\n",                       uk);
 fputs("Command0=beep -f 440 -l 25\n",                                             uk);
 fputs("Command1=beep -f 520 -l 25\n",                                             uk);

 //  
 fclose(uk);

 //    
 //   - ,     sudo 
 //   root    
 char cmd[250];
 sprintf(cmd, "chown %s:%s %s", getUserName(), getUserName(), fileName);
 if(popen(cmd, "r")==NULL)
  {
   printf("Can't run command '%s'\n", cmd);
   return;
  }

}


//        
void Config::createDirIfNotExists(char *dirName, char *userName)
{
 struct stat status;

 if(stat(dirName, &status)!=0)
  {
   //  
   printf("Try create directory %s\n", dirName);
   int result=mkdir(dirName, 0755);

   if(result==0)
    printf("Directory create OK.\n");    
   else
    printf("Error if directory create.\n");    

   //    
   //   - ,     sudo 
   //   root    
   char cmd[250];
   sprintf(cmd, "chown %s:%s %s", userName, userName, dirName);
   if(popen(cmd, "r")==NULL)
    {
     printf("Can't run command '%s'\n", cmd);
     return;
    }
  }
}


//    
char *Config::getUserDirectory()
{
 static char userHomeDir[250];

 char *userHome;

 userHome=getenv("HOME");
 if(userHome==NULL)
  {
   printf("Enviroment variable HOME is not set. Program was closed.\n");
   exit(1);
  }

 sprintf(userHomeDir, userHome); 
 
 return userHomeDir;
}


//   
char *Config::getUserName()
{
 static char userName[250];

 char *value;

 value=getenv("USER");
 if(value==NULL)
  {
   printf("Enviroment variable USER is not set. Program was closed.\n");
   exit(1);
  }

 sprintf(userName, value); 
 
 return userName;
}


char *Config::getInputDevice()
{
 return inputDevice;
}


int Config::getDeviceType()
{
 return deviceType;
}


int Config::getNumberOfLayout()
{
 return numberOfLayout;
}


int Config::getSwitchMethod()
{
 return switchMethod;
}


char *Config::getEventFilter()
{
 return eventFilter.regexp;
}


char *Config::getEventFilterCompile()
{
 return eventFilter.regexpCompile;
}


char *Config::getSequence(int n)
{
 if(n<0 || n>=CODES_LAYOUT_NUMBER) return NULL;

 return sequences[n].regexp;
}


char *Config::getSequenceCompile(int n)
{
 if(n<0 || n>=CODES_LAYOUT_NUMBER) return NULL;

 return sequences[n].regexpCompile;
}


char *Config::getCommand(int n)
{
 if(n<0 || n>=CODES_LAYOUT_NUMBER) return NULL;

 return commands[n].cmdText;
}

==> loloswitcher_0_20.tar.gz: loloswitcher_0_20/Config.cpp <==
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

#include "Config.h"
#include "ParseHelper.h"


Config::Config()
{
 init();
}


Config::~Config()
{

}


void Config::init(void)
{
 // Очистка имени файла конфига
 sprintf(configFileName, "");


 // ------------------------
 // Сброс переменных конфига
 // ------------------------

 // Файл устройства клавиатуры
 sprintf(inputDevice, "");

 // Количество языковых раскладок
 numberOfLayout=0;

 // Метод переключения
 switchMethod=0;

 // Первичный фильтр
 sprintf(eventFilter.regexp, "");
 memset(eventFilter.regexpCompile, 0, REGEXP_COMPILE_SIZE);

 for(int i=0; i<CODES_LAYOUT_NUMBER; i++)
  {
   // Очищаются данные регулярных выражений
   sprintf(sequences[i].regexp, "");
   memset(sequences[i].regexpCompile, 0, REGEXP_COMPILE_SIZE);

   // Очищаются строки команд
   sprintf(commands[i].cmdText, "");
  }
}


// Чтение файла конфига
bool Config::readFile(char *fileName)
{
 init();

 // Открытие файла
 FILE *uk;
 if((uk = fopen (fileName, "rt")) == NULL) // Если файл не был открыт
  return false;

 // Здесь файл открыт на чтение 

 // Запоминается имя файла
 sprintf(configFileName, fileName);


 // Считывается файл
 char readline[255];
 char tmpline[255];
 char tmpline2[255];
 int  i;
 bool remflag,eqflag;

 // Чтение файла по строкам
 while(!feof(uk)){

  // Обнуление строки
  readline[0]='\0';

  // Чтение строки
  fgets(readline, 250, uk);
  alltrim(readline);

  // Проверка, не является ли строка комментарием
  remflag=false;
  if(readline[0]=='#')
   remflag=true;

  // Поиск символа равенства
  eqflag=false;
  for(i=0; i<=strlen(readline); i++)
   if(readline[i]=='=')
    eqflag=true;

  // Если строка содержит присвоение и не является комментарием
  if(eqflag==true && remflag!=true)
  {
   // Копирование строки readline
   for(i=0; readline[i]!='\0'; i++)
    tmpline[i]=readline[i];
   tmpline[i]='\0';
   getparamname(tmpline);

   // Копирование строки readline
   for(i=0;readline[i]!='\0';i++)
    tmpline2[i]=readline[i];
   tmpline2[i]='\0';
   getparamvalue(tmpline2);

   /*
   printf("readline: %s\n", readline);
   printf("tmpline : %s\n", tmpline);
   printf("tmpline2: %s\n", tmpline2);
   */

   // Загрузка переменной InputDevice
   if(strcmp(tmpline, "InputDevice")==0)
    {
     sprintf(inputDevice, "%s", tmpline2);

     if(strlen(inputDevice)==0)
      {
       printf("Detect empty value for inputDevice\n");
       exit(1);
      }
    }


   // Загрузка переменной DeviceType
   if(strcmp(tmpline, "DeviceType")==0)
    {
     deviceType=atoi(tmpline2);

     if(deviceType<0 || deviceType>1)
      {
       printf("Illegal value for DeviceType: %d\n", deviceType);
       exit(1);
      }
    }

   // Загрузка переменной NumberOfLayout
   if(strcmp(tmpline, "NumberOfLayout")==0)
    {
     numberOfLayout=atoi(tmpline2);

     if(numberOfLayout<1 || numberOfLayout>CODES_LAYOUT_NUMBER)
      {
       printf("Illegal value for NumberOfLayout: %d\n", numberOfLayout);
       exit(1);
      }
    }


   // Загрузка переменной SwitchMethod
   if(strcmp(tmpline, "SwitchMethod")==0)
    {
     switchMethod=atoi(tmpline2);

     if(switchMethod<0 || switchMethod>1)
      {
       printf("Illegal value for switchMethod: %d\n", switchMethod);
       exit(1);
      }  
    }


   // Загрузка переменной EventFilter
   if(strcmp(tmpline, "EventFilter")==0)
    {
     // Запоминается текст регулярного выражения
     sprintf(eventFilter.regexp, tmpline2);

     // Компилируется регулярное выражение
     regexpCompiling(eventFilter.regexp, eventFilter.regexpCompile);
    }


   // Загрузка переменных Sequence и Command
   for(int n=0; n<CODES_LAYOUT_NUMBER && n<numberOfLayout; n++)
    {
     // Формируется имя переменной
     char varName[250];
     sprintf(varName, "Sequence%d", n);

     // Считывание переменной Sequence
     if(strcmp(tmpline, varName)==0)
      {
       // Запоминается текст регулярного выражения
       sprintf(sequences[n].regexp, tmpline2);

       // Компилируется регулярное выражение
       regexpCompiling(sequences[n].regexp, sequences[n].regexpCompile);
      }


     // Загрузка переменной Command
     sprintf(varName, "Command%d", n);

     // Считывание переменной
     if(strcmp(tmpline, varName)==0)
      sprintf(commands[n].cmdText, tmpline2); // Запоминается текст команды

    } // Закрылся цикл загрузки переменных Sequence
    
  }
 };

 fclose(uk);

 return true;

}

// Компилирование регулярного выражения во внутреннее представление
void Config::regexpCompiling(char *regexpText, char *regexpCompileData)
{
 int options = 0;
 const char *error;
 int erroffset;
 pcre *regexp;
 
 regexp=pcre_compile(regexpText, options, &error, &erroffset, NULL);

 // Если компиляция была неудачной
 if(!regexp)
  {
   printf("Problem RegExp compile: '%s'\n", regexpText);
   printf("Error: %s\n", error);
   exit(1);
  }


 // Сохранение скомпилированного регулярного выражения 
 int size, sizeResult;
 sizeResult = pcre_fullinfo(regexp, NULL, PCRE_INFO_SIZE, &size);

 if(sizeResult<0)
  {
   printf("Error if call pcre_fullinfo() for get size, %s\n", regexpText);
   exit(1);
  }

 if(size>=REGEXP_COMPILE_SIZE)
  {
   printf("Compile data for regexp too long, %d, %s\n", size, regexpText);
   exit(1);
  }

 // Скомпилированное выражение заносится в память 
 memcpy(regexpCompileData, (char *)regexp, size);
}


// Метод, распечатывающий на экран значения конфига
void Config::print(void)
{
 printf("\n");
 printf("-------------------\n");
 printf("Reading config data from file %s\n", configFileName);
 printf("-------------------\n");

 printf("InputDevice: %s\n", inputDevice);
 printf("DeviceType: %d\n", deviceType);
 printf("NumberOfLayout: %d\n", numberOfLayout);
 printf("SwitchMethod: %d\n", switchMethod);
 printf("EventFilter: %s\n", eventFilter.regexp);

 // Перебираются языковые слои
 for(int n=0; n<CODES_LAYOUT_NUMBER && n<numberOfLayout; n++)
  printf("Sequence%d: %s\n", n, sequences[n].regexp);

 // Перебираются языковые слои
 for(int n=0; n<CODES_LAYOUT_NUMBER && n<numberOfLayout; n++)
  printf("Command%d: %s\n", n, commands[n].cmdText);

 printf("\n");
}


// Метод создания стандартного конфига
void Config::createStandartConfig()
{
 char dirName[250];

 sprintf(dirName, "%s/.config", getUserDirectory());
 createDirIfNotExists(dirName, getUserName());

 sprintf(dirName, "%s/.config/loloswitcher", getUserDirectory());
 createDirIfNotExists(dirName, getUserName());

 // Открытие файла для записи
 char fileName[250];
 sprintf(fileName, "%s/.config/loloswitcher/config.ini", getUserDirectory());

 FILE *uk;
 if((uk = fopen (fileName, "wt")) == NULL)
  {
   // Если файл не был открыт
   printf("Error! Can not create file %s.\n", fileName);
   return;
  }


 // Стандартный код конфига
 fputs("# LoLo Switcher config file\n",                                            uk);
 fputs("\n",                                                                       uk);
 fputs("# ConfigVersion (do not edit this parameter!)\n",                          uk);
 fputs("ConfigVersion=1\n",                                                        uk);
 fputs("\n",                                                                       uk);
 fputs("# Input device\n",                                                         uk);
 fputs("InputDevice=/dev/input/event0\n",                                          uk);
 fputs("\n",                                                                       uk);
 fputs("# Type of device\n",                                                       uk);
 fputs("# 0 - keyboard\n",                                                         uk);
 fputs("# 1 - joystick\n",                                                         uk);
 fputs("DeviceType=0\n",                                                           uk);
 fputs("\n",                                                                       uk);
 fputs("# Total number of language layout\n",                                      uk);
 fputs("# For example:\n",                                                         uk);
 fputs("# if you use ENG and RUS, set 2\n",                                        uk);
 fputs("# if you use ENG, RUS and UKR, set 3\n",                                   uk);
 fputs("NumberOfLayout=2\n",                                                       uk);
 fputs("\n",                                                                       uk);
 fputs("# Language switch method\n",                                               uk);
 fputs("# 0 - cyclic switch\n",                                                    uk);
 fputs("# 1 - direct switch\n",                                                    uk);
 fputs("SwitchMethod=1\n",                                                         uk);
 fputs("\n",                                                                       uk);
 fputs("# RegExp for primary filtering device events\n",                           uk);
 fputs("# For classic keyboard, set value to ^1,[0-9]+,[0-9]+;$\n",                uk);
 fputs("# and this filter enabled only KeyPress, KeyHold and KeyRelease event.\n", uk);
 fputs("# Before research device codes (with option -t1), clear this value.\n",    uk);
 fputs("EventFilter=^1,[0-9]+,[0-9]+;$\n",                                         uk);
 fputs("\n",                                                                       uk);
 fputs("# RegExp with codes for switch language layouts\n",                        uk);
 fputs("# If you use cyclic switch method, set variable Sequence0 only.\n",        uk);
 fputs("# If you use direct switch method, set variable SequenceX\n",              uk);
 fputs("# for each layout (numeric from 0).\n",                                    uk);
 fputs("Sequence0=(?<!1,29,1;|1,29,2;|1,97,1;|1,97,2;|1,56,1;|1,56,2;|1,100,1;|1,100,2;)1,42,1;1,42,0;$\n", uk);
 fputs("Sequence1=(?<!1,29,1;|1,29,2;|1,97,1;|1,97,2;|1,56,1;|1,56,2;|1,100,1;|1,100,2;)1,54,1;1,54,0;$\n", uk);
 fputs("\n",                                                                       uk);
 fputs("# Bash command if language layout switched (optional).\n",                 uk);
 fputs("# Set CommandX for each layout (numeric from 0).\n",                       uk);
 fputs("Command0=beep -f 440 -l 25\n",                                             uk);
 fputs("Command1=beep -f 520 -l 25\n",                                             uk);

 // Файл закрывается
 fclose(uk);

 // Устанавливается правильный хозяин файла
 // Это нужно из-за того, что при запуске от sudo хозяин
 // ставится как root а не как пользователь
 char cmd[250];
 sprintf(cmd, "chown %s:%s %s", getUserName(), getUserName(), fileName);
 if(popen(cmd, "r")==NULL)
  {
   printf("Can't run command '%s'\n", cmd);
   return;
  }

}


// Проверка и содание каталога если его еще нет
void Config::createDirIfNotExists(char *dirName, char *userName)
{
 struct stat status;

 if(stat(dirName, &status)!=0)
  {
   // Создается каталог
   printf("Try create directory %s\n", dirName);
   int result=mkdir(dirName, 0755);

   if(result==0)
    printf("Directory create OK.\n");    
   else
    printf("Error if directory create.\n");    

   // Устанавливается правильный хозяин директории
   // Это нужно из-за того, что при запуске от sudo хозяин
   // ставится как root а не как пользователь
   char cmd[250];
   sprintf(cmd, "chown %s:%s %s", userName, userName, dirName);
   if(popen(cmd, "r")==NULL)
    {
     printf("Can't run command '%s'\n", cmd);
     return;
    }
  }
}


// Выяснение имя каталога пользователя
char *Config::getUserDirectory()
{
 static char userHomeDir[250];

 char *userHome;

 userHome=getenv("HOME");
 if(userHome==NULL)
  {
   printf("Enviroment variable HOME is not set. Program was closed.\n");
   exit(1);
  }

 sprintf(userHomeDir, userHome); 
 
 return userHomeDir;
}


// Выяснение имени пользователя
char *Config::getUserName()
{
 static char userName[250];

 char *value;

 value=getenv("USER");
 if(value==NULL)
  {
   printf("Enviroment variable USER is not set. Program was closed.\n");
   exit(1);
  }

 sprintf(userName, value); 
 
 return userName;
}


char *Config::getInputDevice()
{
 return inputDevice;
}


int Config::getDeviceType()
{
 return deviceType;
}


int Config::getNumberOfLayout()
{
 return numberOfLayout;
}


int Config::getSwitchMethod()
{
 return switchMethod;
}


char *Config::getEventFilter()
{
 return eventFilter.regexp;
}


char *Config::getEventFilterCompile()
{
 return eventFilter.regexpCompile;
}


char *Config::getSequence(int n)
{
 if(n<0 || n>=CODES_LAYOUT_NUMBER) return NULL;

 return sequences[n].regexp;
}


char *Config::getSequenceCompile(int n)
{
 if(n<0 || n>=CODES_LAYOUT_NUMBER) return NULL;

 return sequences[n].regexpCompile;
}


char *Config::getCommand(int n)
{
 if(n<0 || n>=CODES_LAYOUT_NUMBER) return NULL;

 return commands[n].cmdText;
}
